#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>

#define PRINT_DEBUG   //Uncomment this line if you want to print the values on serial monitor

#define SIGNAL_TIMEOUT 500  // This is signal timeout in milli seconds.
#define MAX_MOTOR_SPEED 200

const uint64_t pipeIn = 0xF9E8F0F0E1LL;
RF24 radio(8, 9); 
unsigned long lastRecvTime = 0;


// structura de date trimisa
struct PacketData
{
  byte xAxisValue;    
  byte yAxisValue;
} receiverData;

//Right motor
int enableRightMotor=5; 
int rightMotorPin1=2;
int rightMotorPin2=3;

//Left motor
int enableLeftMotor=6;
int leftMotorPin1=7;
int leftMotorPin2=4;


void setup()
{
  pinMode(enableRightMotor,OUTPUT);
  pinMode(rightMotorPin1,OUTPUT);
  pinMode(rightMotorPin2,OUTPUT);
  
  pinMode(enableLeftMotor,OUTPUT);
  pinMode(leftMotorPin1,OUTPUT);
  pinMode(leftMotorPin2,OUTPUT);

  rotateMotor(0,0); 
    
  radio.begin();
  radio.setDataRate(RF24_250KBPS);
  radio.openReadingPipe(1,pipeIn);
  radio.startListening(); //start the radio receiver 

  Serial.begin(9600);
  #ifdef PRINT_DEBUG
    Serial.begin(9600);
  #endif
}

void loop()
{
    int rightMotorSpeed=0;
    int leftMotorSpeed=0;
    // Check if RF is connected and packet is available 
    if(radio.isChipConnected() && radio.available())
    {
      // se citesc datele
      radio.read(&receiverData, sizeof(PacketData)); 

      //We will receive value as 0 to 254. Center value is 127
      
     // Serial.println(receiverData.xAxisValue);
     // Serial.println(receiverData.yAxisValue);
      if (receiverData.xAxisValue <= 75)       //Move car Forward
      {
        Serial.println("case1");
        rotateMotor(MAX_MOTOR_SPEED, MAX_MOTOR_SPEED);
      }
      else if (receiverData.xAxisValue >= 175)   //Move car Backward
      {
        Serial.println("case2");
        rotateMotor(-MAX_MOTOR_SPEED, -MAX_MOTOR_SPEED);
      }
      else if (receiverData.yAxisValue <= 75)  //Move car Left
      {
        Serial.println("case3");
        rotateMotor(MAX_MOTOR_SPEED, -MAX_MOTOR_SPEED);
      }
      else if (receiverData.yAxisValue >= 175)   //Move car Right
      {
        Serial.println("case4");
        rotateMotor(-MAX_MOTOR_SPEED, MAX_MOTOR_SPEED);
      }
      else                                      //Stop the car
      {
        rotateMotor(0, 0);
      }
      lastRecvTime = millis();  
      
      #ifdef PRINT_DEBUG  
        Serial.println(receiverData.xAxisValue);
        Serial.println(receiverData.yAxisValue);      
      #endif
    }
    else
    {
      //Signal lost. Reset the motor to stop
      unsigned long now = millis();
      if ( now - lastRecvTime > SIGNAL_TIMEOUT) 
      {
        Serial.println("Lost connection");
        analogWrite(enableRightMotor, 0);
        analogWrite(enableLeftMotor, 0);    

        digitalWrite(leftMotorPin1,LOW);
        digitalWrite(leftMotorPin2,LOW);      
 
        digitalWrite(rightMotorPin1,LOW);
        digitalWrite(rightMotorPin2,LOW);      
     }
     if (radio.isChipConnected() == false)
     {
      Serial.println("Chip not connected");
      analogWrite(enableRightMotor, 0);
      analogWrite(enableLeftMotor, 0);    

      digitalWrite(leftMotorPin1,LOW);
      digitalWrite(leftMotorPin2,LOW);      
 
      digitalWrite(rightMotorPin1,LOW);
      digitalWrite(rightMotorPin2,LOW);      
     }
   }
}


void rotateMotor(int rightMotorSpeed, int leftMotorSpeed)
{
  if (rightMotorSpeed < 0)
  {
    digitalWrite(rightMotorPin1,LOW);
    digitalWrite(rightMotorPin2,HIGH);    
  }
  else if (rightMotorSpeed > 0)
  {
    digitalWrite(rightMotorPin1,HIGH);
    digitalWrite(rightMotorPin2,LOW);      
  }
  else
  {
    digitalWrite(rightMotorPin1,LOW);
    digitalWrite(rightMotorPin2,LOW);      
  }
  
  if (leftMotorSpeed < 0)
  {
    digitalWrite(leftMotorPin1,LOW);
    digitalWrite(leftMotorPin2,HIGH);    
  }
  else if (leftMotorSpeed > 0)
  {
    digitalWrite(leftMotorPin1,HIGH);
    digitalWrite(leftMotorPin2,LOW);      
  }
  else
  {
    digitalWrite(leftMotorPin1,LOW);
    digitalWrite(leftMotorPin2,LOW);      
  }  

  analogWrite(enableRightMotor, abs(rightMotorSpeed));
  analogWrite(enableLeftMotor, abs(leftMotorSpeed));    
}